/*
  ͻ
                                  CHAPITRE V                              
  ͹
   Ficher: FXPAL.H                                                        
   Description: Gestion des effets de palette                             
   Auteur: Franck Bauquier                                                
  ͼ
*/


// ͻ
//  Listing des fonctions                                                 
// ͼ
void writeRGB(unsigned char col, unsigned char r,
	      unsigned char g, unsigned char  b);
void GetRGB(unsigned char col, unsigned char &r,
	    unsigned char  &g, unsigned char  &b);

void LoadPalette(unsigned char palette[256][3]);
int BlackComingOut(int temps);
int ImgComingOut(int temps,unsigned char palette[256][3]);
void SuperBlack();
int RGBComingOut(int temps, unsigned char red,
		 unsigned char green, unsigned char blue);


// ͻ
//  writeRGB - Change la palette                                          
// ͼ
void writeRGB(unsigned char col, unsigned char r,
	      unsigned char  g, unsigned char  b)
{
  outp (0x03C8,col); // Envoie la couleur dans le port d'criture
  outp (0x03C9,r);   // Envoie la composante rouge dans le port de status
  outp (0x03C9,g);   // Envoie la composante verte dans le port de status
  outp (0x03C9,b);   // Envoie la composante bleue dans le port de status
}

// ͻ
//  GetRGB - Obtient des informations sur la palette                      
// ͼ
void GetRGB(unsigned char col, unsigned char &r,
	      unsigned char  &g, unsigned char  &b)
{
  outp (0x03C7,col);  // Envoie la couleur dans le port de lecture
  r = inp (0x03C9);   // "r" reoit la composante rouge
  g = inp (0x03C9);   // "g" reoit la composante verte
  b = inp(0x03C9);    // "b" reoit la composante bleue
}



// ͻ
//  LoadPalette - Charge  la palette                                     
// ͼ
void LoadPalette(unsigned char palette[256][3])
{
for (int j=0;j<256;j++) // Boucle sur le nombre de couleurs de la palette
// Charge au fur et  mesure les couleurs de "palette"
writeRGB((unsigned char)j,palette[j][0]/4,palette[j][1]/4,palette[j][2]/4);
}


// ͻ
//  BlackComingOut - Fondue vers la couleur noire                         
// ͼ
int BlackComingOut(int temps)
{
unsigned char r,g,b; // Variables prtes  recevoir des valeur de couleurs
if(kbhit())getch();  // Si une touche a t presse, on la lit

for (int i=0;i<64;i++) // Boucle sur 64
{
if(kbhit()){getch(); return 123;} // Si une touche est frappe, on retourne
				  // quelque chose diffrent de 0
delay(temps);                     // On attend "temps" ms
for (int j=0;j<256;j++)   // Boucle sur le nombre de couleurs de la palette
{
// Lit les composantes de la couleur en cours
GetRGB(j,r,g,b);
if (r>0)r--; // Si la composante rouge n'est pas  0 on la dcrmente de 1
if (g>0)g--; // Si la composante verte n'est pas  0 on la dcrmente de 1
if (b>0)b--; // Si la composante bleue n'est pas  0 on la dcrmente de 1
// Remplace les composantes de la couleur en cours par celles-ci
writeRGB((unsigned char)j,r,g,b);
}// fin for de j
}// fin for de i
return 0;
}// fin function


// ͻ
//  ImgCommingOut - Fondue vers la palette                                
// ͼ
int ImgComingOut(int temps,unsigned char palette[256][3])
{
unsigned char r,g,b; // Variables prtes  recevoir des valeur de couleurs
if(kbhit())getch();  // Si une touche a t presse, on la lit

for (int i=0;i<64;i++) // Boucle sur 64
{
if(kbhit()){getch(); return 123;} // Si une touche est frappe, on retourne
				  // quelque chose diffrent de 0
delay(temps);                     // On attend "temps" ms
for (int j=0;j<256;j++)   // Boucle sur le nombre de couleurs de la palette
{
// Lit les composantes de la couleur en cours
GetRGB(j,r,g,b);
// Si les composantes est diffrente de celle de "palette", on les
// incrmente ou dcremente selon le cas o elles sont suprieur ou
// infrieur  celles de "palette"
if (r<palette[j][0]/4)r++; else if(r>palette[j][0]/4)r--;
if (g<palette[j][1]/4)g++; else if(g>palette[j][1]/4)g--;
if (b<palette[j][2]/4)b++; else if(b>palette[j][2]/4)b--;
// On remplace les vieilles composantes par les nouvelles
writeRGB((unsigned char)j,r,g,b);
}// fin for de j
}// fin for de i
return 0;
}// fin function


// ͻ
//  SuperBlack - Remplit la palette de couleurs noires                    
// ͼ
void SuperBlack()
{
for (int j=0;j<256;j++) // Boucle sur le nombre de couleurs de la palette
// Change la couleur courante en noire
writeRGB((unsigned char)j,0,0,0);
}



// ͻ
//  RGBComingOut - Fondue vers une couleur pralablement dfinie          
// ͼ
int RGBComingOut(int temps, unsigned char red, unsigned char green,
		 unsigned char blue)
{
unsigned char r,g,b; // Variables prtes  recevoir des valeur de couleurs
if(kbhit())getch();  // Si une touche a t presse, on la lit

for (int i=0;i<64;i++) // Boucle sur 64
{
if(kbhit()){getch(); return 123;} // Si une touche est frappe, on retourne
				  // quelque chose diffrent de 0
delay(temps);                     // On attend "temps" ms
for (int j=0;j<256;j++)   // Boucle sur le nombre de couleurs de la palette
{
// Lit les composantes de la couleur en cours
GetRGB(j,r,g,b);
// Si les composantes sont diffrentes de celles demandes, on les
// incrmente ou dcremente selon le cas o elles sont suprieur ou
// infrieur  celles demandes.
if (r<red)r++; else if(r>red)r--;
if (g<green)g++; else if(g>green)g--;
if (b<blue)b++; else if(b>blue)b--;
writeRGB((unsigned char)j,(unsigned char)r,(unsigned char)g,(unsigned char)b);
}// fin for de j
}// fin for de i
return 0;
}// fin function


